@@ -210,7 +210,8 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+      if (!MagickMonitorFormatted(y,image->rows,exception,LoadImageText,
+                                  image->filename))
         break;
   }
   }
@@ -251,9 +252,10 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
 */
 ModuleExport void RegisterJBIGImage(void)
 {
-#define JBIGDescription  "Joint Bi-level Image experts Group interchange format"
+  static const char
+    *description="Joint Bi-level Image experts Group interchange format";
 
-  char
+  static char
     version[MaxTextExtent];
 
   MagickInfo
@@ -269,30 +271,35 @@ ModuleExport void RegisterJBIGImage(void)
   entry->encoder=(EncoderHandler) WriteJBIGImage;
 #endif
   entry->adjoin=False;
-  entry->description=AcquireString(JBIGDescription);
+  entry->description=description;
   if (*version != '\0')
-    entry->version=AcquireString(version);
-  entry->module=AcquireString("JBIG");
+    entry->version=version;
+  entry->module="JBIG";
+  entry->coder_class=PrimaryCoderClass;
   (void) RegisterMagickInfo(entry);
+
   entry=SetMagickInfo("JBG");
 #if defined(HasJBIG)
   entry->decoder=(DecoderHandler) ReadJBIGImage;
   entry->encoder=(EncoderHandler) WriteJBIGImage;
 #endif
-  entry->description=AcquireString(JBIGDescription);
+  entry->description=description;
   if (*version != '\0')
-    entry->version=AcquireString(version);
-  entry->module=AcquireString("JBIG");
+    entry->version=version;
+  entry->module="JBIG";
+  entry->coder_class=PrimaryCoderClass;
   (void) RegisterMagickInfo(entry);
+
   entry=SetMagickInfo("JBIG");
 #if defined(HasJBIG)
   entry->decoder=(DecoderHandler) ReadJBIGImage;
   entry->encoder=(EncoderHandler) WriteJBIGImage;
 #endif
-  entry->description=AcquireString(JBIGDescription);
+  entry->description=description;
   if (*version != '\0')
-    entry->version=AcquireString(version);
-  entry->module=AcquireString("JBIG");
+    entry->version=version;
+  entry->module="JBIG";
+  entry->coder_class=PrimaryCoderClass;
   (void) RegisterMagickInfo(entry);
 }
 
@@ -442,7 +449,8 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
           q+=export_info.bytes_exported;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+              if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                          SaveImageText,image->filename))
                 break;
         }
     }
@@ -487,7 +495,9 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (!MagickMonitor(SaveImagesText,scene++,GetImageListLength(image),&image->exception))
+    if (!MagickMonitorFormatted(scene++,GetImageListLength(image),
+                                &image->exception,SaveImagesText,
+                                image->filename))
       break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
