@@ -245,7 +245,6 @@ static int BlobClose(jas_stream_obj_t *object)
     *source = (StreamManager *) object;
 
   CloseBlob(source->image);
-  MagickFreeMemory(source);
   return (0);
 }
 
@@ -558,7 +557,8 @@ static Image *ReadJP2Image(const ImageInfo *image_info,
         break;
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+          if (!MagickMonitorFormatted(y,image->rows,exception,LoadImageText,
+                                      image->filename))
             break;
     }
   if (number_components == 1)
@@ -643,8 +643,8 @@ ModuleExport void RegisterJP2Image(void)
     *entry;
 
   entry=SetMagickInfo("JP2");
-  entry->description=AcquireString("JPEG-2000 JP2 File Format Syntax");
-  entry->module=AcquireString("JP2");
+  entry->description="JPEG-2000 JP2 File Format Syntax";
+  entry->module="JP2";
   entry->magick=(MagickHandler) IsJP2;
   entry->adjoin=False;
   entry->seekable_stream=True;
@@ -653,11 +653,12 @@ ModuleExport void RegisterJP2Image(void)
   entry->decoder=(DecoderHandler) ReadJP2Image;
   entry->encoder=(EncoderHandler) WriteJP2Image;
 #endif
+  entry->coder_class=StableCoderClass;
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("JPC");
-  entry->description=AcquireString("JPEG-2000 Code Stream Syntax");
-  entry->module=AcquireString("JPC");
+  entry->description="JPEG-2000 Code Stream Syntax";
+  entry->module="JPC";
   entry->magick=(MagickHandler) IsJPC;
   entry->adjoin=False;
   entry->seekable_stream=True;
@@ -666,11 +667,12 @@ ModuleExport void RegisterJP2Image(void)
   entry->decoder=(DecoderHandler) ReadJP2Image;
   entry->encoder=(EncoderHandler) WriteJP2Image;
 #endif
+  entry->coder_class=StableCoderClass;
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("PGX");
-  entry->description=AcquireString("JPEG-2000 VM Format");
-  entry->module=AcquireString("PGX");
+  entry->description="JPEG-2000 VM Format";
+  entry->module="PGX";
   entry->magick=(MagickHandler) IsJPC;
   entry->adjoin=False;
   entry->seekable_stream=True;
@@ -679,6 +681,7 @@ ModuleExport void RegisterJP2Image(void)
   /* Don't set an encoder since PGX is not a standard format */
   entry->decoder=(DecoderHandler) ReadJP2Image;
 #endif
+  entry->coder_class=StableCoderClass;
   (void) RegisterMagickInfo(entry);
 
   /*
@@ -959,7 +962,8 @@ static unsigned int WriteJP2Image(const ImageInfo *image_info,Image *image)
       }
     if (image->previous == (Image *) NULL)
       if (QuantumTick(y,image->rows))
-        if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+        if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                    SaveImageText,image->filename))
           break;
   }
   (void) strlcpy(magick,image_info->magick,MaxTextExtent);
