@@ -1028,7 +1028,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
         status=MagickFail;
         break;
       }
-    indexes=GetIndexes(image);
+    indexes=AccessMutableIndexes(image);
 
     /* FIXME .... */
     if (jpeg_read_scanlines(&jpeg_info,scanline,1) != 1)
@@ -1090,7 +1090,8 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
         break;
       }
     if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+      if (!MagickMonitorFormatted(y,image->rows,exception,LoadImageText,
+                                  image->filename))
         {
           status=MagickFail;
           break;
@@ -1158,9 +1159,17 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
 */
 ModuleExport void RegisterJPEGImage(void)
 {
+  static char
+    version[MaxTextExtent];
+
   MagickInfo
     *entry;
 
+  version[0]='\0';
+#if defined(JPEG_LIB_VERSION)
+  FormatString(version,"%d",JPEG_LIB_VERSION);
+#endif
+
   entry=SetMagickInfo("JPEG");
   entry->thread_support=False; /* libjpeg is not thread safe */
 #if defined(HasJPEG)
@@ -1169,18 +1178,10 @@ ModuleExport void RegisterJPEGImage(void)
 #endif
   entry->magick=(MagickHandler) IsJPEG;
   entry->adjoin=False;
-  entry->description=
-    AcquireString("Joint Photographic Experts Group JFIF format");
-#if defined(JPEG_LIB_VERSION)
-  {
-    char
-      version[MaxTextExtent];
-
-    FormatString(version,"%d",JPEG_LIB_VERSION);
-    entry->version=AcquireString(version);
-  }
-#endif
-  entry->module=AcquireString("JPEG");
+  entry->description="Joint Photographic Experts Group JFIF format";
+  if (version[0] != '\0')
+    entry->module="JPEG";
+  entry->coder_class=PrimaryCoderClass;
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("JPG");
@@ -1190,9 +1191,11 @@ ModuleExport void RegisterJPEGImage(void)
   entry->encoder=(EncoderHandler) WriteJPEGImage;
 #endif
   entry->adjoin=False;
-  entry->description=
-    AcquireString("Joint Photographic Experts Group JFIF format");
-  entry->module=AcquireString("JPEG");
+  entry->description="Joint Photographic Experts Group JFIF format";
+  if (version[0] != '\0')
+    entry->version=version;
+  entry->module="JPEG";
+  entry->coder_class=PrimaryCoderClass;
   (void) RegisterMagickInfo(entry);
 }
 
@@ -1679,7 +1682,7 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
       /* FIXME: density should not be set via image_info->density
          but removing this support may break some applications. */
       count=GetMagickDimension(image_info->density,&image->x_resolution,
-        &image->y_resolution);
+        &image->y_resolution,NULL,NULL);
       if (count == 1 )
         image->y_resolution=image->x_resolution;
     }
@@ -2017,7 +2020,8 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
                 }
               (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
               if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+                if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                            SaveImageText,image->filename))
                   break;
             }
         }
@@ -2043,7 +2047,8 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
                   }
                 (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
                 if (QuantumTick(y,image->rows))
-                  if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+                  if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                              SaveImageText,image->filename))
                     break;
               }
           }
@@ -2071,7 +2076,8 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
                   }
                 (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
                 if (QuantumTick(y,image->rows))
-                  if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+                  if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                              SaveImageText,image->filename))
                     break;
               }
           }
@@ -2106,7 +2112,8 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
               }
             (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
             if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+              if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                          SaveImageText,image->filename))
                 break;
           }
       }
@@ -2132,7 +2139,8 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
                 }
               (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
               if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+                if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                            SaveImageText,image->filename))
                   break;
             }
         }
@@ -2160,7 +2168,8 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
                 }
               (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
               if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+                if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                            SaveImageText,image->filename))
                   break;
             }
         }
