@@ -38,13 +38,13 @@
 */
 #include "magick/studio.h"
 #include "magick/attribute.h"
-#include "magick/cache.h"
 #include "magick/composite.h"
 #include "magick/constitute.h"
 #include "magick/decorate.h"
 #include "magick/gem.h"
 #include "magick/monitor.h"
 #include "magick/montage.h"
+#include "magick/pixel_cache.h"
 #include "magick/render.h"
 #include "magick/resize.h"
 #include "magick/utility.h"
@@ -83,10 +83,10 @@ MagickExport MontageInfo *CloneMontageInfo(const ImageInfo *image_info,
   MontageInfo
     *clone_info;
 
-  clone_info=(MontageInfo *) AcquireMemory(sizeof(MontageInfo));
+  clone_info=MagickAllocateMemory(MontageInfo *,sizeof(MontageInfo));
   if (clone_info == (MontageInfo *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-      "UnableToAllocateMontageInfo");
+    MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+      UnableToAllocateMontageInfo);
   GetMontageInfo(image_info,clone_info);
   if (montage_info == (MontageInfo *) NULL)
     return(clone_info);
@@ -111,7 +111,7 @@ MagickExport MontageInfo *CloneMontageInfo(const ImageInfo *image_info,
   clone_info->border_color=montage_info->border_color;
   clone_info->matte_color=montage_info->matte_color;
   clone_info->gravity=montage_info->gravity;
-  (void) strncpy(clone_info->filename,montage_info->filename,MaxTextExtent-1);
+  (void) strlcpy(clone_info->filename,montage_info->filename,MaxTextExtent);
   return(clone_info);
 }
 
@@ -143,18 +143,18 @@ MagickExport void DestroyMontageInfo(MontageInfo *montage_info)
   assert(montage_info != (MontageInfo *) NULL);
   assert(montage_info->signature == MagickSignature);
   if (montage_info->geometry != (char *) NULL)
-    LiberateMemory((void **) &montage_info->geometry);
+    MagickFreeMemory(montage_info->geometry);
   if (montage_info->tile != (char *) NULL)
-    LiberateMemory((void **) &montage_info->tile);
+    MagickFreeMemory(montage_info->tile);
   if (montage_info->title != (char *) NULL)
-    LiberateMemory((void **) &montage_info->title);
+    MagickFreeMemory(montage_info->title);
   if (montage_info->frame != (char *) NULL)
-    LiberateMemory((void **) &montage_info->frame);
+    MagickFreeMemory(montage_info->frame);
   if (montage_info->texture != (char *) NULL)
-    LiberateMemory((void **) &montage_info->texture);
+    MagickFreeMemory(montage_info->texture);
   if (montage_info->font != (char *) NULL)
-    LiberateMemory((void **) &montage_info->font);
-  LiberateMemory((void **) &montage_info);
+    MagickFreeMemory(montage_info->font);
+  MagickFreeMemory(montage_info);
 }
 
 /*
@@ -190,7 +190,7 @@ MagickExport void GetMontageInfo(const ImageInfo *image_info,
   assert(image_info->signature == MagickSignature);
   assert(montage_info != (MontageInfo *) NULL);
   (void) memset(montage_info,0,sizeof(MontageInfo));
-  (void) strncpy(montage_info->filename,image_info->filename,MaxTextExtent-1);
+  (void) strlcpy(montage_info->filename,image_info->filename,MaxTextExtent);
   montage_info->geometry=AllocateString(DefaultTileGeometry);
   montage_info->gravity=CenterGravity;
   montage_info->tile=AllocateString("6x4");
@@ -258,8 +258,8 @@ static int SceneCompare(const void *x,const void *y)
 MagickExport Image *MontageImages(const Image *images,
   const MontageInfo *montage_info,ExceptionInfo *exception)
 {
-#define MontageImageText  "  Create visual image directory...  "
-#define TileImageText  "  Create image tiles...  "
+#define MontageImageText  "[%s] Create visual image directory..."
+#define TileImageText  "[%s] Create image tiles..."
 
   char
     tile_geometry[MaxTextExtent],
@@ -347,9 +347,10 @@ MagickExport Image *MontageImages(const Image *images,
   number_images=GetImageListLength(images);
   master_list=ImageListToArray(images,exception);
   if (master_list == (Image **) NULL)
-    ThrowImageException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToCreateImageMontage");
+    ThrowImageException3(ResourceLimitError,MemoryAllocationFailed,
+      UnableToCreateImageMontage);
   image_list=master_list;
+  thumbnail=(Image *) NULL;
   for (i=0; i < (long) number_images; i++)
   {
     handler=SetMonitorHandler((MonitorHandler) NULL);
@@ -372,7 +373,8 @@ MagickExport Image *MontageImages(const Image *images,
       break;
     image_list[i]=thumbnail;
     (void) SetMonitorHandler(handler);
-    if (!MagickMonitor(TileImageText,i,number_images,&image->exception))
+    if (!MagickMonitorFormatted(i,number_images,&image->exception,
+                                TileImageText,image->filename))
       break;
   }
   if (i < (long) number_images)
@@ -380,10 +382,10 @@ MagickExport Image *MontageImages(const Image *images,
       if (!thumbnail)
         --i;
 
-      for (tile=0; tile <= i; tile++)
+      for (tile=0; tile <= (unsigned long) i; tile++)
         if (image_list[tile])
           DestroyImage(image_list[tile]);
-      LiberateMemory((void **) &master_list);
+      MagickFreeMemory(master_list);
       return((Image *) NULL);
     }
   /*
@@ -474,12 +476,12 @@ MagickExport Image *MontageImages(const Image *images,
   draw_info->stroke=montage_info->stroke;
   draw_info->fill=montage_info->fill;
   draw_info->text=AllocateString("");
-  GetTypeMetrics(image_list[0],draw_info,&metrics);
+  (void) GetTypeMetrics(image_list[0],draw_info,&metrics);
   texture=(Image *) NULL;
   if (montage_info->texture != (char *) NULL)
     {
-      (void) strncpy(image_info->filename,montage_info->texture,
-        MaxTextExtent-1);
+      (void) strlcpy(image_info->filename,montage_info->texture,
+        MaxTextExtent);
       texture=ReadImage(image_info,exception);
     }
   /*
@@ -542,10 +544,10 @@ MagickExport Image *MontageImages(const Image *images,
     /*
       Initialize montage image.
     */
-    (void) strncpy(montage->filename,montage_info->filename,MaxTextExtent-1);
+    (void) strlcpy(montage->filename,montage_info->filename,MaxTextExtent);
     montage->columns=bounds.width;
     montage->rows=bounds.height;
-    SetImage(montage,OpaqueOpacity);
+    (void) SetImage(montage,OpaqueOpacity);
     /*
       Set montage geometry.
     */
@@ -553,11 +555,11 @@ MagickExport Image *MontageImages(const Image *images,
     count=1;
     for (tile=0; tile < tiles_per_page; tile++)
       count+=strlen(image_list[tile]->filename)+1;
-    montage->directory=(char *) AcquireMemory(count);
+    montage->directory=MagickAllocateMemory(char *,count);
     if ((montage->montage == (char *) NULL) ||
         (montage->directory == (char *) NULL))
-      ThrowImageException(ResourceLimitError,"MemoryAllocationFailed",
-        "UnableToCreateImageMontage");
+      ThrowImageException3(ResourceLimitError,MemoryAllocationFailed,
+        UnableToCreateImageMontage);
     x_offset=0;
     y_offset=(long) title_offset;
     FormatString(montage->montage,"%ldx%ld%+ld%+ld",
@@ -568,13 +570,13 @@ MagickExport Image *MontageImages(const Image *images,
     *montage->directory='\0';
     for (tile=0; tile < tiles_per_page; tile++)
     {
-      (void) strncat(montage->directory,image_list[tile]->filename,
-        MaxTextExtent-strlen(montage->directory)-1);
+      (void) strlcat(montage->directory,image_list[tile]->filename,
+        MaxTextExtent);
       (void) strcat(montage->directory,"\n");
     }
     handler=SetMonitorHandler((MonitorHandler) NULL);
     if (texture != (Image *) NULL)
-      TextureImage(montage,texture);
+      (void) TextureImage(montage,texture);
     if (montage_info->title != (char *) NULL)
       {
         char
@@ -696,7 +698,7 @@ MagickExport Image *MontageImages(const Image *images,
                   break;
                 for (columns=0; columns < Min(tile_info.x,4); columns++)
                 {
-                  Modulate(0.0,0.0,-25.0+4*columns,&q->red,&q->green,&q->blue);
+                  Modulate(0.0,0.0,53.0,&q->red,&q->green,&q->blue);
                   q++;
                 }
                 if (!SyncImagePixels(montage))
@@ -710,7 +712,7 @@ MagickExport Image *MontageImages(const Image *images,
                   break;
                 for (columns=0; columns < (long) image->columns; columns++)
                 {
-                  Modulate(0.0,0.0,-25.0+4*rows,&q->red,&q->green,&q->blue);
+                  Modulate(0.0,0.0,53.0,&q->red,&q->green,&q->blue);
                   q++;
                 }
                 if (!SyncImagePixels(montage))
@@ -750,7 +752,8 @@ MagickExport Image *MontageImages(const Image *images,
       DestroyImage(image);
       DestroyImage(image_list[tile]);
       (void) SetMonitorHandler(handler);
-      if (!MagickMonitor(MontageImageText,tiles,total_tiles,&image->exception))
+      if (!MagickMonitorFormatted(tiles,total_tiles,&image->exception,
+                                  MontageImageText,image->filename))
         break;
       tiles++;
     }
@@ -773,7 +776,7 @@ MagickExport Image *MontageImages(const Image *images,
   DestroyImage(tile_image);
   if (texture != (Image *) NULL)
     DestroyImage(texture);
-  LiberateMemory((void **) &master_list);
+  MagickFreeMemory(master_list);
   DestroyDrawInfo(draw_info);
   DestroyImageInfo(image_info);
   while (montage->previous != (Image *) NULL)
