@@ -230,10 +230,11 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
         errno=status;
         return((SemaphoreInfo *) NULL);
       }
-
+#if 0
 #if defined(PTHREAD_MUTEX_RECURSIVE)
     (void) pthread_mutexattr_settype(&mutexattr, PTHREAD_MUTEX_RECURSIVE);
 #endif /* PTHREAD_MUTEX_RECURSIVE */
+#endif
 
     if ((status=pthread_mutex_init(&semaphore_info->mutex,&mutexattr)) != 0)
       {
@@ -446,7 +447,7 @@ MagickExport MagickPassFail LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 {
   assert(semaphore_info != (SemaphoreInfo *) NULL);
   assert(semaphore_info->signature == MagickSignature);
-
+  /* fprintf(stderr,"lock %p\n", semaphore_info); */
 #if defined(HAVE_PTHREAD)
   {
     int
@@ -460,7 +461,9 @@ MagickExport MagickPassFail LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
     if ((semaphore_info->lock_depth > 0) &&
         (semaphore_info->owner_thread_id == self))
       {
-        fprintf(stderr,"Warning: recursive semaphore lock detected!\n");
+        fprintf(stderr,
+                "Warning: recursive semaphore lock detected (depth=%u p=%p)!\n",
+                semaphore_info->lock_depth,semaphore_info);
         fflush(stderr);
       }
     
@@ -512,7 +515,6 @@ MagickExport MagickPassFail UnlockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 {
   assert(semaphore_info != (SemaphoreInfo *) NULL);
   assert(semaphore_info->signature == MagickSignature);
-
   if (semaphore_info->lock_depth == 0)
     {
       fprintf(stderr,
@@ -522,11 +524,15 @@ MagickExport MagickPassFail UnlockSemaphoreInfo(SemaphoreInfo *semaphore_info)
       return (MagickFail);
     }
 
+  /* fprintf(stderr,"unlock %p\n", semaphore_info); */
+
   semaphore_info->lock_depth--;
-  
+
 #if defined(HAVE_PTHREAD)
   /* Enforce that unlocking thread is the same as the locking thread */
   assert(pthread_equal(semaphore_info->owner_thread_id,pthread_self()));
+  if (semaphore_info->lock_depth == 0)
+    semaphore_info->owner_thread_id=-1;
   if (pthread_mutex_unlock(&semaphore_info->mutex))
     return(MagickFail);
 #endif
